@@ -98,10 +98,12 @@ class MapWindow(Ui_Form):
         self.jet  = cm.get_cmap("jet",50) #colormap
         self.timeseries = {}
         self.arrivals = {}
-        self.colors = itertools.cycle([Qt.Qt.red,Qt.Qt.green,Qt.Qt.blue,Qt.Qt.cyan, Qt.Qt.magenta,Qt.Qt.yellow,Qt.Qt.black])
+        self.colors = itertools.cycle([Qt.Qt.red,Qt.Qt.green,Qt.Qt.blue,Qt.Qt.cyan, Qt.Qt.magenta,Qt.Qt.yellow,Qt.Qt.black, Qt.Qt.darkCyan,  Qt.Qt.darkRed,  Qt.Qt.darkGreen,  Qt.Qt.darkBlue,  Qt.Qt.darkMagenta,  Qt.Qt.darkYellow])
         self.setupQwtPlot()
         self.step = 0
         self.M = None #initialize map widget
+        # Configuring View
+        self.mapView.setViewportUpdateMode(QtGui.QGraphicsView.SmartViewportUpdate)
         
         # Overloading event-handling methods for self.mapView
         self.mapView.keyPressEvent = MethodType(keyPressEvent, self.mapView)
@@ -112,6 +114,7 @@ class MapWindow(Ui_Form):
         QtCore.QObject.connect(self.horizontalSlider,QtCore.SIGNAL("valueChanged()"), self.on_horizontalSlider_valueChanged)
         QtCore.QObject.connect(self.pushButton,QtCore.SIGNAL("released()"), self.replay)
         QtCore.QObject.connect(self.splitter,QtCore.SIGNAL("splitterMoved()"), self.centerScene)
+        QtCore.QObject.connect(self.showSumBox,QtCore.SIGNAL("stateChanged(int)"), self.on_showSumBox_stateChanged)
 #        self.server = MapServer()
 #        self.server.map = self.M
 #        st = threading.Thread(target=self.server.start)
@@ -119,7 +122,7 @@ class MapWindow(Ui_Form):
         
     def setupQwtPlot(self):
         """
-        sets up the time series plot
+        Sets up the time series plot
         """
 #        self.qwtPlot.setTitle('%s'%self.variable)
         self.qwtPlot.setAxisTitle(Qwt.QwtPlot.xBottom, 'time')
@@ -133,9 +136,52 @@ class MapWindow(Ui_Form):
         self.mX.setXValue(0)
         self.mX.attach(self.qwtPlot)
         self.qwtPlot.replot()
+        
+    def fill_nodel_list(self, pl):
+        """
+        creates check boxes on the node list group box
+        pl is a list of polygons
+        """
+        lo = self.nodeListLayout
+        pl.sort(key=lambda p:p.name)#adding sites in alphabetical order
+        for n in pl:
+            cb = QtGui.QCheckBox(self.scrollArea)
+            cb.setText(n.name)
+            QtCore.QObject.connect(cb,QtCore.SIGNAL("stateChanged(int)"), n.select)
+            cb.setToolTip(str(n.geocode))
+            lo.addWidget(cb)
+#        self.nodeGroupBox.setLayout(lo)
+            
+    def add_global_ts_curve(self, scope="all"):
+        """
+        Plots a time series which is the sum of all(scope="all") or 
+        selected timeseries (scope="set")
+        """
+        t = self.timeseries.keys()
+        t.sort()
+        data = [0]*len(self.timeseries)
+        if scope == "all":
+            name = "Total"
+            for k, v in self.timeseries.iteritems():
+                data[k] = sum(v.values())
+#                print v
+        elif scope == "Selected":
+            name ="Sum of Selected"
+            #TODO: fix this
+#        print data
+        curve = Qwt.QwtPlotCurve(name)
+        pen = Qt.QPen(Qt.Qt.gray)
+        pen.setStyle(4)
+        pen.setWidth(3)
+        curve.setPen(pen)
+        curve.attach(self.qwtPlot)
+        curve.setData(t, data)
+        self.globalCurve = curve
+        self.qwtPlot.replot()
+    
     def addTsCurve(self, gc, name):
         """
-        plots a time series curve to the plot window
+        Plots a time series curve to the plot window
         """
         
         data = [0]*len(self.timeseries)
@@ -161,8 +207,8 @@ class MapWindow(Ui_Form):
         xmax,ymax = self.M.xmax, self.M.ymax
         xxs = (xmax-xmin)*1.1 #percentage of extra space
         yxs = (ymax-ymin)*1.1 #percentage of extra space
+        
         #calculating center of scene
-
         xc = (xmax+xmin)/2. 
         yc = (ymax+ymin)/2.
         self.mapView.scene = QtGui.QGraphicsScene(self.mapView)
@@ -182,14 +228,16 @@ class MapWindow(Ui_Form):
         
         for p in self.M.polyList:
             self.mapView.scene.addItem(p)
+        self.fill_nodel_list(self.M.polyList)
         #self.scene.addText("%s,%s,%s,%s"%(xmin, xxs, ymin, yxs))
-        self.polys = [item for item in self.mapView.scene.items() if isinstance(item, Polygon)]
+#        self.polys = [item for item in self.mapView.scene.items() if isinstance(item, Polygon)]
         #self.mapView.addGraph(self.polys)
-        self.mapView.setMinimumSize(400, 400)
+        self.mapView.setMinimumSize(300, 300)
         #self.mapView.setWindowTitle(self.tr("Network View"))
         scale_factor = self.mapView.width()/xxs
         self.mapView.scale(scale_factor, scale_factor)
-        
+        if self.showSumBox.isChecked():
+            self.add_global_ts_curve("all")
         #print self.polys
     
     def drawGraph(self, nlist, elist=[] ):
@@ -228,7 +276,7 @@ class MapWindow(Ui_Form):
         
     def centerScene(self):
         """
-        centers the scene and fits the specified rectangle to it
+        Centers the scene and fits the specified rectangle to it
         """
         ymax, ymin = self.ymax, self.ymin
         xmax, xmin = self.xmax, self.xmin
@@ -262,9 +310,12 @@ class MapWindow(Ui_Form):
         if max(datadict.values()) > 1:
             normw = max(datadict.values())
         else:
-            normw = 1
+            normw = 1.
         for gc, val in datadict.iteritems():
-            val /= normw #normalize values if necessary
+            try:
+                val /= normw #normalize values if necessary
+            except TypeError as e:
+                print gc, val, datadict[gc],  e
             col = self.jet(val)#rgba list
             gc = int(gc)
 #            print gc, type(gc)
@@ -285,10 +336,7 @@ class MapWindow(Ui_Form):
         QtCore.QObject.connect(rw,QtCore.SIGNAL("flash"), self.flashBorders)
         QtCore.QObject.connect(self.pushButton_2,QtCore.SIGNAL("released()"), stop_replay)
         rw.render()
-        
 
-            
-    
     def drawStep(self,step,  datadict={}):
         """
         Draws one timestep on the map
@@ -320,6 +368,7 @@ class MapWindow(Ui_Form):
     def on_horizontalSlider_valueChanged(self):
         if self.horizontalSlider.isEnabled():
             self.on_horizontalSlider_sliderMoved()
+    
     def on_horizontalSlider_sliderMoved(self):
         """
         Handles updating the display on a slider move
@@ -332,6 +381,19 @@ class MapWindow(Ui_Form):
         self.mX.setLabel(Qwt.QwtText("t = %s"%self.step))
         self.qwtPlot.replot()
         
+#    @QtCore.pyqtSignature("int")
+    def on_showSumBox_stateChanged(self, st):
+        """
+        Handles adding/remove global timeseries curve
+        """
+        if st:
+#            print "adding global series"
+            self.add_global_ts_curve()
+        else:
+#            print "removing global series"
+            self.globalCurve.detach()
+            self.qwtPlot.replot()
+        
 #BlackBox  :-)
 class BaseBox(object):
     def __init__(self, *args, **kwargs):
@@ -587,9 +649,6 @@ class BaseMap(object):
         else:
             print "shapefile %s not found in %s"%(fname, os.getcwd())
 
-    
-
-
     def Reader(self, fname):
         """
         Reads shapefiles vector files.
@@ -654,6 +713,9 @@ class QtMap(BaseMap):
         BaseMap.__init__(self, fname,namefield,geocfield)
         
     def dbound(self, pol, geocode = None , name=""):
+        """
+        Draws a polygon.
+        """
         #FIXME: consertar algoritmo para funcionar com qualquer sistema de coordenadas
         p = Polygon(pol, geocode,name,  self.display)
         self.xmin = p.xmin if p.xmin<self.xmin else self.xmin
@@ -688,6 +750,7 @@ class Polygon(QtGui.QGraphicsItem):
         self.geocode = geocode
         self.name = name
         self.setToolTip(str(self.geocode)+ " - "+name)
+        self.setCursor(QtCore.Qt.PointingHandCursor)
         self.setFlag(QtGui.QGraphicsItem.ItemIsSelectable)
         self.setZValue(1)
     
@@ -710,6 +773,13 @@ class Polygon(QtGui.QGraphicsItem):
 #        print button
         scenepos = event.scenePos()
         pos = event.pos()
+        self.select()
+        #QtGui.QGraphicsItem.mousePressEvent(self, event)
+        
+    def select(self):
+        """
+        Toggle selection of polygon
+        """
         if self.isSelected():
 #            print "unselect"
             self.setSelected(False)
@@ -727,9 +797,7 @@ class Polygon(QtGui.QGraphicsItem):
             self.display.addTsCurve(self.geocode, self.name)
             self.display.qwtPlot.replot()
         self.update()
-        #QtGui.QGraphicsItem.mousePressEvent(self, event)
         
-
     def mouseReleaseEvent(self, event):
         self.update()
         #QtGui.QGraphicsItem.mouseReleaseEvent(self, event)
