@@ -15,19 +15,13 @@ except ImportError:
     epigrassLogger.error("Please install PyQT 4")
     print "Please install PyQT 4"
 
-from Epigrass.manager import *
+from manager import *
 import threading,  subprocess,  glob
-from Epigrass.Ui_cpanel4new import Ui_MainWindow
-from Epigrass.Ui_about4 import Ui_aboutDialog as aboutDialog
+from Ui_cpanel4new import Ui_MainWindow
+from Ui_about4 import Ui_aboutDialog as aboutDialog
 import sys,ConfigParser, string, copy, commands,getpass
-import Epigrass.epiplay as epi
-from Epigrass import spread,  dgraph
-try:
-    import psyco
-    psyco.full()
-except:
-    epigrassLogger.info("Install Psyco for better performance." )
-    pass
+import epiplay as epi
+import spread,  dgraph
 
 
 class MainWindow_Impl(QtGui.QMainWindow, Ui_MainWindow):
@@ -115,7 +109,6 @@ class MainWindow_Impl(QtGui.QMainWindow, Ui_MainWindow):
         Initializes the epigrassrc file.
         """
         usern = getpass.getuser() #Get the user name
-        os.mknod(self.epigrassrc)
         f = open(self.epigrassrc,'w')
         
         rcskel = ['#EpiGrass Configuration File',
@@ -150,7 +143,6 @@ class MainWindow_Impl(QtGui.QMainWindow, Ui_MainWindow):
         
         if os.access(self.epigrassrc,os.F_OK):
             os.remove(self.epigrassrc)
-            os.mknod(self.epigrassrc)
         f = open(self.epigrassrc,'w')
         
         rcskel = ['#EpiGrass Configuration File',
@@ -340,7 +332,7 @@ an editor and your model's script."""),
             
             # In case of a batch
             if S.Batch:
-                self.textEdit1.insertPlainText('Simulation Started.')
+                self.textEdit1.insertPlainText(self.trUtf8('Simulation Started.'))
                 
                 # run the batch list
                 for i in S.Batch:
@@ -450,42 +442,72 @@ epigrass.sql > mysql"""),
                 self.trUtf8("""Could not open the report.
 Make sure you have generated it."""),
                 self.trUtf8("&OK"))
-            
-    
-    
+
+
+    def find_data_dirs(self):
+        print "scanning"
+        basedir = self.rootdir
+        os.chdir(basedir)
+        datadirs = [] #list of  outdata directories
+        while datadirs == []:
+            basedir = str(QFileDialog.getExistingDirectory(\
+                None,
+                self.trUtf8("Select a Directory"),
+                QtCore.QString(),
+                QFileDialog.Options(QFileDialog.ShowDirsOnly)))
+            os.chdir(basedir)
+            datadirs = [d for d in glob.glob("outdata-*") if os.path.isdir(d)] #list of  outdata directories
+
+
+        return basedir, datadirs
+
     def onVisual(self):
         """
         Scan the epigrass database an shows available simulations.
         """
-        basedir = self.rootdir
-        if not self.dbType.currentIndex():
+        basedir, datadirs = self.find_data_dirs()
+        if self.dbType.currentIndex() == 0:
             if not self.pwEdit.text():
                 QMessageBox.warning(None,
                     self.trUtf8("Missing Database Password"),
                     self.trUtf8("""Please enter the password for the MySQL database in the first tab."""))
                 return
         # for SQLite databases check for the existence os the database file.
-        if self.dbType.currentIndex() == 1: 
-            if not os.path.exists('Epigrass.sqlite'):
-                basedir = str(QFileDialog.getExistingDirectory(\
+        elif self.dbType.currentIndex() == 1:
+            print "sqlite"
+            if len(datadirs) > 1:
+                datadir = str(QtGui.QInputDialog.getItem(\
                     None,
-                    self.trUtf8("No SQLite database found. Please select a new directory."),
-                    QtCore.QString(),
-                    QFileDialog.Options(QFileDialog.ShowDirsOnly)))
-            if not glob.glob(basedir+'*.sqlite'):
-                QMessageBox.information(None,
-                    self.trUtf8("No Database found"),
-                    self.trUtf8("""No file ending with ".sqlite" was found. 
-Please try a different  directory."""),
-                    QMessageBox.StandardButtons(\
-                        QMessageBox.Ok),
-                    QMessageBox.Ok)
+                    self.trUtf8("Please Select Database to Open"),
+                    self.trUtf8("Database"),
+                    datadirs,
+                    0, False)[0])
+            elif len(datadirs) == 0:
+                QtGui.QMessageBox.critical(None,
+                    self.trUtf8("Invalid Folder"),
+                    self.trUtf8("""This directory does not contain a model outdata subdirectory. """),
+                    QtGui.QMessageBox.StandardButtons(\
+                        QtGui.QMessageBox.Abort))
+
             else:
+                datadir = datadirs[0]
+            print basedir
+            print datadir
+            fulldatadir = os.path.join(basedir,datadir)
+            os.chdir(fulldatadir)
+            print os.getcwd()
+            if not os.path.exists('Epigrass.sqlite'):
+                QtGui.QMessageBox.warning(None,
+                    self.trUtf8("No Database found"),
+                    self.trUtf8("""Please try again"""),
+                    QtGui.QMessageBox.StandardButtons(\
+                        QtGui.QMessageBox.Ok))
                 os.chdir(basedir)
+                print os.getcwd()
                 
         self.Display=epi.viewer(host=str(self.hostnEdit.text()),port=int(str(self.portEdit.text())),
                             db='epigrass',user=str(self.uidEdit.text()), pw=str(self.pwEdit.text()),backend=self.conf['database.backend'], gui=self)
-        os.chdir(self.curdir)
+        os.chdir(fulldatadir)
 
         for s in self.Display.tables:
             if s.endswith('_meta'):
@@ -523,7 +545,7 @@ Please try a different  directory."""),
         else:
             mapa =None
         modname = table.split('_')[0] #self.sim.modelName.split('/')[-1]
-        #change the the outdata directory
+        #change to the outdata directory
         if not os.path.split(os.getcwd())[1].startswith("outdata"):
             os.chdir("outdata-"+modname)
         nodes,am = self.Display.readNodes(modname, table)
@@ -537,7 +559,7 @@ Please select another table from the menu."""))
         numbnodes = len(nodes)
         data = self.Display.readData(table)
         edata = self.Display.readEdges(table)
-        os.chdir(self.rootdir)
+#        os.chdir(self.rootdir)
         numbsteps = len(data)/numbnodes
         self.Display.viewGraph(nodes,am,var,mapa)
         self.Display.anim(data,edata,numbsteps,pos,r)
