@@ -3,42 +3,36 @@
 Model Management and simulation objects.
 """
 import cPickle, time
-import os, codecs,  subprocess
+import os, codecs, subprocess
 from copy import deepcopy
 from xmlrpclib import ServerProxy, Error
 from simobj import graph, edge, siteobj
 from numpy import *
+import numpy as np
 import spread
+from collections import OrderedDict
 from numpy.random import uniform, poisson
-from sqlobject import *
-from math import *
 from data_io import *
 from optparse import OptionParser
 import ConfigParser
 import string, sys, getpass
 import report as Rp
 import epigdal
+import json
 import __version__
-import encodings.utf_8
-import encodings.latin_1
-import dataObject as DO
 import sqlite3, MySQLdb
 #import pycallgraph
 
-# Import Psyco if available
-#try:
-#    import psyco
-#
-#    psyco.full()
-#except ImportError:
-#    pass
+
+
 
 class simulate:
     """
     This class takes care of setting up the model, simulating it, and storing the results
     """
-    def __init__(self, fname=None, host='localhost',port=3306,db='epigrass',user='epigrass', password='epigrass',backend = 'mysql'):
+    def __init__(self, fname=None, host='localhost',port=3306,db='epigrass',user='epigrass', password='epigrass',backend = 'sqlite', silent=False):
         #pycallgraph.start_trace()
+#        self.pool = multiprocessing.Pool()
         self.host = host
         self.port = port
         self.usr = user
@@ -50,6 +44,7 @@ class simulate:
         self.shapefile = None
         self.World = None
         self.shpout = True
+        self.silent = silent
         self.dir = os.getcwd()
         sys.path.insert(0,self.dir) #add current path to sys.path
         self.gui = 0 # True if this object was created by the gui
@@ -64,18 +59,14 @@ class simulate:
         if self.shapefile: #create world object if a shapefile is provided
             self.World = epigdal.World(*self.shapefile+[self.outdir])
         self.chkScript()
-        self.encoding = 'latin-1'#default encoding for strings in the sites and edges files
+        self.encoding = 'utf-8'#default encoding for strings in the sites and edges files
         self.round = 0 #No replicates
         sitios = loadData(self.sites, sep=',', encoding=self.encoding)
         ed = loadData(self.edges,sep=',', encoding=self.encoding)
         l = self.instSites(sitios)
         e = self.instEdges(l,ed)
         self.g = self.instGraph(self.modelName,1,l,e)
-        #if self.layer:
-        #    pass
-            #self.grassVect2ascii(self.layer)
 
-        #self.g.plotDegreeDist()
 
 
     def loadModelScript(self,fname):
@@ -154,7 +145,7 @@ class simulate:
             self.Batch = []#Turns off Batch mode
             self.round = 0# Initialize replicate counter
     # generate dictionaries for parameters and inits
-        self.inits = {}
+        self.inits = OrderedDict()
         self.parms = {}
         for k,v in config.items():
             if k.startswith('initial conditions'):
@@ -202,21 +193,27 @@ class simulate:
         Here the site specific events are passed to each site, and the site models are created.
         """
         header = sitelist.pop(0)
+        ncols = len(header)
         objlist=[]
         for site in sitelist:
+            if len(site) != ncols:
+                raise ValueError("This line in your sites file has a different number elements:\n%s"%str(site))
             if ':' in site[0]:
                 lat = self.deg2dec(site[0])
                 long = self.deg2dec(site[1])
             else:
                 lat = float(site[0])
                 long = float(site[1])
-            objlist.append(siteobj(site[2],site[3],(lat,long),int(strip(site[4])),tuple([float(strip(i)) for i in site[5:]])))
+            objlist.append(siteobj(site[2],site[3],(lat,long),int(strip(site[4])),tuple([float(i.strip()) for i in site[5:]])))
         for o in objlist:
-            if self.stochTransp: o.stochtransp = 1
+            if self.stochTransp:
+                o.stochtransp = 1
+            else:
+                o.stochtransp = 0
             N = o.totpop #local copy for reference on model creation
             values = o.values #local copy for reference on model creation
-            inits={}
-            parms={}
+            inits = OrderedDict()
+            parms = {}
             #eval parms and inits
             for k,v in self.inits.items():
                 inits[k] = eval(v)
@@ -262,7 +259,6 @@ class simulate:
                         o.createModel((E,I,S),self.modtype,self.modelName,v=values,bi=inits,bp=parms)
             else:
                 o.createModel((E,I,S),self.modtype,self.modelName,v=values,bi=inits,bp=parms)
-
         return objlist
 
     def instEdges(self,sitelist, edgelist):
@@ -327,16 +323,17 @@ class simulate:
         option: if 1 randomize; if 2, return unrundomized sequence
         """
         if option==1:
-            poplist = [log10(site.totpop) for site in self.g.site_list]
+            poplist = [log10(site.totpop) for site in self.g.site_dict.itervalues()]
             lpl = len(poplist)
             popprob = array(poplist)/sum(poplist) #acceptance probability
             u = floor(uniform(0,lpl, self.replicas))
             sites=[]
             i=0
+            site_list = self.g.site_list
             while i < self.replicas:
                 p = uniform(0,1)
                 if p <= popprob[int(u[i])]:
-                    sites.append(self.g.site_list[int(u[i])])
+                    sites.append(site_list[int(u[i])])
                     i += 1
         elif option ==2:
             sites = self.g.site_list
@@ -351,9 +348,10 @@ class simulate:
         #inits[self.seed[0][1].lower()] += 1
         seedvar = self.seed[0][1].lower() #retrieve the name of the variable containing the seeds
         self.Say('seedvar= %s'%seedvar)
-
+        site_list = self.g.site_list
         self.seed =[(seed.geocode, seedvar,n)]
-        for site in self.g.site_list:
+
+        for site in site_list:
             if site.geocode == seed.geocode:
                 site.bi[seedvar] = n
 #                site.ts[0][seedpos] = n
@@ -370,9 +368,6 @@ class simulate:
         if '/' in self.modelName:
             self.modelName = os.path.split(self.modelName)[-1]
 #        print "====>", self.gui
-#        if self.gui and not self.round: #open the display window only on the first round of a reprun
-#            subprocess.Popen(['python','Epigrass/dgraph.py'])
-#            self.dserver = ServerProxy("http://localhost:50000", allow_none = True) # local display server
         time.sleep(10)
         self.Say('Simulation starting.')
         start = time.time()
@@ -387,14 +382,10 @@ class simulate:
         if self.shpout:
             self.outToShp()
         self.dumpData()
-        spread.Spread(self.g,  self.outdir, self.encoding)
+        spread.Spread(self.g, self.outdir, self.encoding)
         #self.saveModel(self.modelName)
         #pycallgraph.make_dot_graph(self.modelName+'_callgraph.png')
-        #print len(self.g.site_list[-1].ts)
 
-##        for i in self.g.site_list:
-##            i.plotItself()
-##        show()
 
     def createDataFrame(self,site):
         """
@@ -431,39 +422,69 @@ class simulate:
             return
         #Initialize world output layers
         self.Say("REading Nodes from shapefile...")
-        self.World.getNodeList(self.World.ds.GetLayerByName(self.World.layerlist[0]))
+        self.World.get_node_list(self.World.ds.GetLayerByName(self.World.layerlist[0]))
         self.Say("Done reading nodes!")
         self.Say("Creating Nodes shapefile...")
-        self.World.createNodeLayer()
+        self.World.create_node_layer()
         self.Say("Done creating Nodes shapefile!")
-        elist = [(e.source.geocode,e.dest.geocode,sum(e.ftheta),sum(e.btheta)) for e in self.g.edge_list]
+        elist = [(gcs[0],gcs[1],sum(e.ftheta),sum(e.btheta)) for gcs,e in self.g.edge_dict.iteritems()]
         self.Say("Creating Edges shapefile...")
-        self.World.createEdgeLayer(elist)
+        self.World.create_edge_layer(elist)
         self.Say("Done creating Edges shapefile!")
         #Generate site epidemic stats
         varlist = ["prevalence","totalcases", "arrivals","population"]
-        sitestats = []
-        names = {}
-        for site in self.g.site_list:
-            names[int(site.geocode)] = site.sitename
-            prevalence = float(site.totalcases)/site.totpop
-            #print prevalence
-            sitestats.append((site.geocode,prevalence,site.totalcases,sum(site.thetahist),float(site.totpop)))
+        sitestats = [(site.geocode,float(site.totalcases)/site.totpop,site.totalcases,sum(site.thetahist),float(site.totpop)) for site in self.g.site_dict.itervalues()]
+        names = {k: v.sitename for k,v in self.g.site_dict.iteritems()}
 
         self.Say("Creating Data shapefile...")
-        self.World.createDataLayer(varlist,sitestats)
+        self.World.create_data_layer(varlist,sitestats)
         self.Say("Done creating Data shapefile!")
         #Generate the kml too.
-        self.Say("Creating KML output file...")
+        self.out_to_kml(names)
+
+        self.Say("Done creating KML files!")
+        #close files
+        self.World.close_sources()
+    
+    def out_to_kml(self, names):
+        """
+        Generates output to kml files
+        """
+        self.Say("Creating KML output files...")
         lr = self.World.datasource.GetLayer(0)
         k = epigdal.KmlGenerator()
         k.addNodes(lr,names)
         k.writeToFile(self.outdir)
-        self.Say("Done creating KML file!")
-        #close files
-        self.World.closeSources()
-
+        site_list = self.g.site_list
+        site_dict = self.g.site_dict
+# Temporarily disabled animation output due to the sheer size of the kmz files
+#        if  len(site_dict)*len(site_dict.values()[0].ts)<10000: #Only reasonably sized animations
+#            for i, v in enumerate(site_dict.values()[0].vnames):
+#                ka = epigdal.AnimatedKML(os.path.join(self.outdir, 'Data.kml'), extrude = True)
+#                data = [(str(site.geocode), t, p[i]) for site in site_dict.itervalues() for t, p in enumerate(site.ts)]
+#                ka.add_data(data)
+#                ka.save(v+'_animation')
+#                self.Say(v+'_animation')
+#                del ka
+#        else:
+#            self.Say("Simulation too large to export as kml.")
+        
+        self.Say("Done creating KML files!")
 
+    def series_to_JSON(self):
+        """
+        Saves timeseries to JSON for uploading to epigrass web
+        """
+        self.Say('Saving series to JSON')
+        series = {}
+        for gc,s in self.g.site_dict.iteritems():
+            ts = np.array(s.ts)
+            sdict = {}
+            for i,vn in enumerate(s.vnames):
+                sdict[vn] = ts[:,i].tolist()
+            series[gc] = sdict
+        with open('series.json','w') as f:
+            json.dump(series,f)
 
     def writeMetaCSV(self, table):
         """
@@ -492,7 +513,7 @@ class simulate:
         head = ['geocode','time','totpop','name',
         'lat','longit']
         headerwritten = False
-        for site in self.g.site_list:
+        for site in self.g.site_dict.itervalues():
             t = 0
             regb = [str(site.geocode),str(t),str(site.totpop),
                 site.sitename.replace('"','').encode('ascii','replace'),
@@ -537,145 +558,145 @@ class simulate:
         g.close()
         os.chdir(self.dir)
 
-    def outToODb(self,table,mode='b'):
-        """
-        Insert simulation results in a database using sqlobject.
-        if mode = b, do batch inserts
-        if mode = p, do parallel inserts
-        """
-        #set connection parameters
-        try:
-            DO.Connect(self.backend.lower(),self.usr,self.passw,self.host,self.port,self.db)
-            self.Say('Saving data on %s database...'%self.backend)
-        except:
-            self.Say('Database Connection Failed')
-        #basic connection variables
-        name = table+'_'+self.now
-        self.outtable = name
-
-        DO.Site.sqlmeta.table = name
-        DO.Edge.sqlmeta.table = name+'e'
-        if self.backend.lower()=='sqlite':
-            DO.Site.sqlmeta.createSQL = {'sqlite':['PRAGMA synchronous=OFF;','PRAGMA default_cache_size=12000000;']}
-            DO.Site._connection.commit()
-            DO.Edge.sqlmeta.createSQL = {'sqlite':['PRAGMA synchronous=OFF;','PRAGMA default_cache_size=12000000;']}
-            DO.Edge._connection.commit()
-        #adding columns to sites table
-        try: #Only add if they don't exist. On Batch runs this will catch the add error
-            DO.Site.sqlmeta.addColumn(StringCol('incidence'))
-            DO.Site.sqlmeta.addColumn(StringCol('arrivals'))
-        except: pass
-        if self.g.site_list[0].values:#add columns for 'values' variables
-            for i in xrange(len(self.g.site_list[0].values)):
-                try:
-                    DO.Site.sqlmeta.addColumn(StringCol('values%s'%i))
-                except: pass
-        for s in self.g.site_list[0].vnames: #add columns for state variables
-            try:
-                DO.Site.sqlmeta.addColumn(StringCol(s))
-            except: pass
-        #creating tables
-        DO.Site.createTable()
-        DO.Edge.createTable()
-        #saving pickle of adjacency matrix
-        matname = 'adj_'+self.modelName # table
-        adjfile = open(matname,'w')
-        cPickle.dump(self.g.getConnMatrix(),adjfile)
-        adjfile.close()
-        #calling insert function
-        if mode == 'b':
-            self.batchInsert()
-        elif mode == 'p':
-            self.parInsert()
-
-    def batchInsert(self):
-        """
-        Do the inserts all at once. After the simulation is done
-        """
-        #inserting data in sites table
-        for site in self.g.site_list:
-            dicin={
-            'geocode':site.geocode,
-            'lat':float(site.pos[0]),
-            'longit':float(site.pos[1]),
-            'name':site.sitename.replace('"','').encode('ascii','replace'),
-            'totpop':int(site.totpop)}
-            if site.values:
-                for n,v in enumerate(site.values):
-                    #print v,type(v)
-                    dicin['values%s'%n] = str(v)
-            ts = array(site.ts[1:]) #remove init conds so that ts and inc are the same size
-            t = 0
-            for i in ts:
-                dicin['incidence']= str(site.incidence[t])
-                dicin['arrivals'] = str(site.thetahist[t])
-                dicin['time'] = t
-                for n,v in enumerate(site.vnames):
-                    #print i[n],type( i[n])
-                    dicin[v] = str(i[n])
-                    DO.Site(**dicin)
-                t += 1
-        #DO.Site._connection.commit()
-        #inserting data in edges table
-        for e in self.g.edge_list:
-            rlist = []
-            edicin={
-            'source_code':e.source.geocode,
-            'dest_code':e.dest.geocode}
-            t=0
-            for f,b in zip(e.ftheta,e.btheta):
-                edicin['time'] = t
-                edicin['ftheta'] = f
-                edicin['btheta'] = b
-                rlist.append(edicin)
-                t += 1
-            insobj = DO.sqlbuilder.Insert(DO.Edge.sqlmeta.table,valueList=rlist)
-            DO.Edge._connection.queryAll(DO.Edge._connection.sqlrepr(insobj))
-            #DO.Edge._connection.commit()
-
-    def parInsert(self):
-        """
-        do the insertions in a separate process.
-        """
-        pid = os.fork()
-        if pid:
-            #parent process get on with the simulation
-            return
-        else:
-        #child process inserts data in sites table
-            time.sleep(1+self.g.simstep)
-            for site in self.g.site_list:
-                dicin={
-                'geocode':site.geocode,
-                'lat':float(site.pos[0]),
-                'longit':float(site.pos[1]),
-                'name':site.sitename.replace('"','').encode('ascii','replace'),
-                'totpop':int(site.totpop)}
-                if site.values:
-                    for n,v in enumerate(site.values):
-                        #print v,type(v)
-                        dicin['values%s'%n] = str(v)
-                ts = array(site.ts[1:]) #remove init conds so that ts and inc are the same size
-                t = self.g.simstep
-                dicin['incidence']= str(site.incidence[t])
-                dicin['arrivals'] = str(site.thetahist[t])
-                dicin['time'] = t
-                for n,v in enumerate(site.vnames):
-                    #print i[n],type( i[n])
-                    dicin[v] = str(i[n])
-                DO.Site(**dicin)
-            DO.Site._connection.commit()
-            #inserting data in edges table
-            for e in self.g.edge_list:
-                edicin={
-                'source_code':e.source.geocode,
-                'dest_code':e.dest.geocode}
-                edicin['time'] = t
-                edicin['ftheta'] = e.ftheta[t]
-                edicin['btheta'] = e.btheta[t]
-                DO.Edge(**edicin)
-            DO.Edge._connection.commit()
-            sys.exit("commit successful")
+#    def outToODb(self,table,mode='b'):
+#        """
+#        Insert simulation results in a database using sqlobject.
+#        if mode = b, do batch inserts
+#        if mode = p, do parallel inserts
+#        """
+#        #set connection parameters
+#        try:
+#            DO.Connect(self.backend.lower(),self.usr,self.passw,self.host,self.port,self.db)
+#            self.Say('Saving data on %s database...'%self.backend)
+#        except:
+#            self.Say('Database Connection Failed')
+#        #basic connection variables
+#        name = table+'_'+self.now
+#        self.outtable = name
+#
+#        DO.Site.sqlmeta.table = name
+#        DO.Edge.sqlmeta.table = name+'e'
+#        if self.backend.lower()=='sqlite':
+#            DO.Site.sqlmeta.createSQL = {'sqlite':['PRAGMA synchronous=OFF;','PRAGMA default_cache_size=12000000;']}
+#            DO.Site._connection.commit()
+#            DO.Edge.sqlmeta.createSQL = {'sqlite':['PRAGMA synchronous=OFF;','PRAGMA default_cache_size=12000000;']}
+#            DO.Edge._connection.commit()
+#        #adding columns to sites table
+#        try: #Only add if they don't exist. On Batch runs this will catch the add error
+#            DO.Site.sqlmeta.addColumn(StringCol('incidence'))
+#            DO.Site.sqlmeta.addColumn(StringCol('arrivals'))
+#        except: pass
+#        if self.g.site_list[0].values:#add columns for 'values' variables
+#            for i in xrange(len(self.g.site_list[0].values)):
+#                try:
+#                    DO.Site.sqlmeta.addColumn(StringCol('values%s'%i))
+#                except: pass
+#        for s in self.g.site_list[0].vnames: #add columns for state variables
+#            try:
+#                DO.Site.sqlmeta.addColumn(StringCol(s))
+#            except: pass
+#        #creating tables
+#        DO.Site.createTable()
+#        DO.Edge.createTable()
+#        #saving pickle of adjacency matrix
+#        matname = 'adj_'+self.modelName # table
+#        adjfile = open(matname,'w')
+#        cPickle.dump(self.g.getConnMatrix(),adjfile)
+#        adjfile.close()
+#        #calling insert function
+#        if mode == 'b':
+#            self.batchInsert()
+#        elif mode == 'p':
+#            self.parInsert()
+#
+#    def batchInsert(self):
+#        """
+#        Do the inserts all at once. After the simulation is done
+#        """
+#        #inserting data in sites table
+#        for site in self.g.site_list:
+#            dicin={
+#            'geocode':site.geocode,
+#            'lat':float(site.pos[0]),
+#            'longit':float(site.pos[1]),
+#            'name':site.sitename.replace('"','').encode('ascii','replace'),
+#            'totpop':int(site.totpop)}
+#            if site.values:
+#                for n,v in enumerate(site.values):
+#                    #print v,type(v)
+#                    dicin['values%s'%n] = str(v)
+#            ts = array(site.ts[1:]) #remove init conds so that ts and inc are the same size
+#            t = 0
+#            for i in ts:
+#                dicin['incidence']= str(site.incidence[t])
+#                dicin['arrivals'] = str(site.thetahist[t])
+#                dicin['time'] = t
+#                for n,v in enumerate(site.vnames):
+#                    #print i[n],type( i[n])
+#                    dicin[v] = str(i[n])
+#                    DO.Site(**dicin)
+#                t += 1
+#        #DO.Site._connection.commit()
+#        #inserting data in edges table
+#        for e in self.g.edge_list:
+#            rlist = []
+#            edicin={
+#            'source_code':e.source.geocode,
+#            'dest_code':e.dest.geocode}
+#            t=0
+#            for f,b in zip(e.ftheta,e.btheta):
+#                edicin['time'] = t
+#                edicin['ftheta'] = f
+#                edicin['btheta'] = b
+#                rlist.append(edicin)
+#                t += 1
+#            insobj = DO.sqlbuilder.Insert(DO.Edge.sqlmeta.table,valueList=rlist)
+#            DO.Edge._connection.queryAll(DO.Edge._connection.sqlrepr(insobj))
+#            #DO.Edge._connection.commit()
+#
+#    def parInsert(self):
+#        """
+#        do the insertions in a separate process.
+#        """
+#        pid = os.fork()
+#        if pid:
+#            #parent process get on with the simulation
+#            return
+#        else:
+#        #child process inserts data in sites table
+#            time.sleep(1+self.g.simstep)
+#            for site in self.g.site_list:
+#                dicin={
+#                'geocode':site.geocode,
+#                'lat':float(site.pos[0]),
+#                'longit':float(site.pos[1]),
+#                'name':site.sitename.replace('"','').encode('ascii','replace'),
+#                'totpop':int(site.totpop)}
+#                if site.values:
+#                    for n,v in enumerate(site.values):
+#                        #print v,type(v)
+#                        dicin['values%s'%n] = str(v)
+#                ts = array(site.ts[1:]) #remove init conds so that ts and inc are the same size
+#                t = self.g.simstep
+#                dicin['incidence']= str(site.incidence[t])
+#                dicin['arrivals'] = str(site.thetahist[t])
+#                dicin['time'] = t
+#                for n,v in enumerate(site.vnames):
+#                    #print i[n],type( i[n])
+#                    dicin[v] = str(i[n])
+#                DO.Site(**dicin)
+#            DO.Site._connection.commit()
+#            #inserting data in edges table
+#            for e in self.g.edge_list:
+#                edicin={
+#                'source_code':e.source.geocode,
+#                'dest_code':e.dest.geocode}
+#                edicin['time'] = t
+#                edicin['ftheta'] = e.ftheta[t]
+#                edicin['btheta'] = e.btheta[t]
+#                DO.Edge(**edicin)
+#            DO.Edge._connection.commit()
+#            sys.exit("commit successful")
 
     def writeMetaTable(self, table):
         """
@@ -740,12 +761,12 @@ class simulate:
                 con = sqlite3.connect("Epigrass.sqlite")
                 os.chdir(self.dir)
             # Define number of variables to be stored
-            nvar = len(self.g.site_list[0].ts[-1]) +4 #state variables,  plus coords, plus incidence, plus infected arrivals.
+            nvar = len(self.g.site_dict.values()[0].ts[-1]) +4 #state variables,  plus coords, plus incidence, plus infected arrivals.
             str1 = '`%s` FLOAT(9),'*nvar #nvar variables in the table
             str1lite = '%s REAL,'*nvar #nvar variables in the SQLite table
-            varnames = ['lat','longit']+list(self.g.site_list[0].vnames)+['incidence']+['Arrivals']
+            varnames = ['lat','longit']+list(self.g.site_dict.values()[0].vnames)+['incidence']+['Arrivals']
 #            print nvar, varnames, str1
-            str1 = str1[:len(str1)-1] % tuple(varnames) #insert variable names (MySQL)
+            str1 = str1[:-1] % tuple(varnames) #insert variable names (MySQL)
             str1lite = str1lite[:len(str1lite)-1] % tuple(varnames) #insert variable names (SQLITE)
             Cursor = con.cursor()
             str2 = """CREATE TABLE %s(
@@ -771,7 +792,7 @@ class simulate:
                 str3 = str3[:-1]+')'
             sql2 = 'INSERT INTO %s' % table + ' VALUES('+str3
             nvalues = []
-            for site in self.g.site_list:
+            for site in self.g.site_dict.itervalues():
                 geoc = site.geocode
                 lat = site.pos[0]
                 long = site.pos[1]
@@ -807,9 +828,9 @@ class simulate:
                 Cursor.execute(esqlite)
                 esql2 = 'INSERT INTO %s' % etable + ' VALUES(?,?,?,?,?)'
             values = []
-            for e in self.g.edge_list:
-                s = e.source.geocode
-                d = e.dest.geocode
+            for gcs,e in self.g.edge_dict.iteritems():
+                s = gcs[0]
+                d = gcs[1]
                 t=0
                 for f,b in zip(e.ftheta,e.btheta):
                     values.append((s,d,t,f,b))
@@ -829,7 +850,7 @@ class simulate:
 
     def criaAdjMatrix(self):
         # saving the adjacency  matrix
-        codeslist = [str(i.geocode) for i in self.g.site_list]
+        codeslist = [str(i.geocode) for i in self.g.site_dict.itervalues()]
         if not os.path.exists('adjmat.csv'):
             self.Say('Saving the adjacency  matrix...')
             am = self.g.getConnMatrix()
@@ -845,16 +866,12 @@ class simulate:
         """
         Dumps data as csv (comma-separated-values)
         """
-##        pid = os.fork()
-##        if pid:
-##            return
-##        else:
         self.Say("Starting simulation Analysis")
         curdir = os.getcwd()
         if not self.outdir == curdir:
             os.chdir(self.outdir)
 
-        codeslist = [str(i.geocode) for i in self.g.site_list]
+        codeslist = self.g.site_dict.keys()
         self.criaAdjMatrix()
         #saving the shortest path matrices
 #        if not os.path.exists('spmat.csv'):
@@ -887,23 +904,25 @@ class simulate:
         epp.write('time,site,infector\n')
         for i in self.g.epipath:
             #print i
+            site = self.g.site_dict[i[1]]
             infectors = i[-1]
             # sorting infectors by number of infective contributed
             if len(infectors):
-                reverse_infectors = [ [v[1],v[0]] for v in infectors.items()]
-                reverse_infectors.sort()
-                mli = [reverse_infectors[j][1] for j in xrange(0,len(reverse_infectors))][-1].sitename#Most likely infector
+                reverse_infectors = infectors.items()
+                reverse_infectors.sort(key=lambda t:t[1])
+                mli = reverse_infectors[-1][0].sitename#Most likely infector
             else:
                 mli = 'NA'
             #print i[1].sitename, type(i[1].sitename), mli
-            epp.write(str(i[0])+','+i[1].sitename+','+mli+'\n')
+            epp.write(str(i[0])+','+site.sitename+','+mli+'\n')
         epp.close()
         self.Say('Done!')
+        self.g.save_topology('network.graphml')
 
         #saving Epistats
         self.Say('Saving Epidemiological results...')
         stats = [str(i) for i in self.g.getEpistats()]
-        seed = [s for s in self.g.site_list if s.geocode == self.seed[0][0]][0]
+        seed = [s for s in self.g.site_dict.itervalues() if s.geocode == self.seed[0][0]][0]
         stats.pop(1) #Remove epispeed which is a vector
         if os.path.exists('epistats.csv'):
             stf = codecs.open('epistats.csv','a',self.encoding)#append to file
@@ -933,7 +952,7 @@ class simulate:
 
         #sitef.write('round,geocode,name,infection_time,degree,centrality,betweeness,theta_index,distance,seed,seedname\n')
         sitef.write('round,geocode,name,infection_time,degree,seed,seedname\n')
-        for s in self.g.site_list:
+        for s in self.g.site_dict.itervalues():
             degree = str(s.getDegree())
             #central = str(s.getCentrality())
             #bet = str(s.getBetweeness())
@@ -951,6 +970,8 @@ class simulate:
 #            sitef.write(str(self.round)+','+str(s.geocode)+','+s.sitename+','+it+','+degree+','+central+','+bet+','+thidx+','+distseed+','+seedgc+','+seedname+'\n')
             sitef.write(str(self.round)+','+str(s.geocode)+','+s.sitename+','+it+','+degree+','+seedgc+','+seedname+'\n')
 
+        # Saving series to JSON
+        self.series_to_JSON()
         os.chdir(curdir)
 
         self.Say('Done saving data!')
@@ -970,10 +991,6 @@ class simulate:
         g = cPickle.load(fname)
         return g
 
-    def fins(self):
-        """
-        call parallel inserts
-        """
 
     def runGraph(self,graphobj,iterations=1, transp=0):
         """
@@ -981,33 +998,28 @@ class simulate:
         """
         g = graphobj
         g.maxstep = iterations
-        sites = tuple(graphobj.site_list)
-        edges = tuple(graphobj.edge_list)
-        #viewer = Viewer(g,self.shapefile[0], "geocode" )
+        sites = graphobj.site_dict.values()
+        edges = graphobj.edge_dict.values()
+#        g.sites_done = 0
+        parallel = 0
+
         if transp:
             for n in xrange(iterations):
-                for i in sites:
-                    i.runModel()
+#                print "==> ",g.simstep
+                results=[i.runModel() for i in sites]
+                [r.wait() for r in results]
                 for j in edges:
-                    j.transportStoD()
-                    j.transportDtoS()
-##                self.outToODb(self.modelName,mode='p')
+                    j.migrate()
                 if self.gui:
-#                    self.gui.graphDisplay.drawStep(g.simstep, dict([(s.geocode, s.incidence[-1]) for s in sites]))
                     self.gui.stepLCD.display(g.simstep)
                     self.gui.app.processEvents()
-#                    self.gui.RT.mutex.lock()
-#                    self.gui.RT.emit(self.gui.QtCore.SIGNAL("drawStep"), g.simstep, dict([(s.geocode, s.incidence[-1]) for s in sites]))
-#                    self.gui.RT.mutex.unlock()
                 g.simstep += 1
+                g.sites_done = 0
         else:
             for n in xrange(iterations):
                 for i in sites:
                     i.runModel()
-##                self.outToODb(self.modelName,mode='p')
-                #viewer.show(n)
                 if self.gui:
-                    #self.gui.graphDisplay.drawStep(g.simstep, dict([(s.geocode, s.incidence[-1]) for s in sites]))
                     self.gui.stepLCD.display(g.simstep)
                     self.gui.app.processEvents()
                     self.gui.RT.mutex.lock()
@@ -1020,7 +1032,11 @@ class simulate:
         """
         if self.gui:
             self.gui.textEdit1.insertPlainText(string+'\n''')
-        print string+'\n'
+        else:
+            if self.silent:
+                return
+            print string+'\n'
+
 def storeSimulation(s,db='epigrass', host='localhost',port=3306):
     """
     store the Simulate object *s* in the epigrass database
@@ -1038,59 +1054,6 @@ def storeSimulation(s,db='epigrass', host='localhost',port=3306):
     Cursor.execute(sql2,blob)
     con.close()
 
-class Bunch:
-    """
-    Class to store a bunch os variables
-    """
-    def __init__(self, **kw):
-       self.__dict__.update(kw)
-
-class Tree:
-    """
-    Tree object representing the spread of an epidemic
-    """
-    def __init__(self,Simulation):
-        self.g = Simulation.g
-        Self.S = Simulation
-        pass
-
-    def writeNexus(self):
-        """
-        Writes a Nexus file containing(.nex) the phylogeographical
-        tree of the epidemic.
-        """
-        ntax = len(self.g.epipath)
-        sitenames = [i[1].sitename for i in self.g.epipath]
-        Name = self.S.modelName
-        header = """#NEXUS
-        [written %s from model % by EpiGrass]
-        """%(time.asctime(),Name)
-        taxablock = """BEGIN TAXA;
-                            TITLE %s;
-                            DIMENSIONS NTAX=%s;
-                            TAXLABELS
-                                %s
-                                ;
-                        END;
-
-        """%(Name,ntax,' '.join(sitenames))
-
-        treeblock = """BEGIN TREES;
-                        Title Epidemic_Path;
-                        LINK Taxa = %s;"""%(Name,)
-
-        transtring = ','.join(['%s %s'%(i,j) for i,j in zip(range(1,len(sitenames)+1),sitenames)])+';'
-        treeblock2 = """TRANSLATE
-                            %s
-                            """%transtring
-        venn = self.getVenn()
-        treeblock3 = """TREE 'Default ladder+' = %;
-                        END;"""%venn
-    def getVenn(self):
-        """"
-        Generates Venn Diagram from epipath.
-        """
-        pass
 
 def onStraightRun(options, args):
     """
